/*
*===========================================================
*        File Name: cmd.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cmd.h"
#include "uart.h"
#include "cn0352.h"
#include "adc.h"

#define CONST_DWORD_ALIGNMENT				0

static void cmd_wr(void);
static void cmd_rd(unsigned char *str, unsigned char length);
static void cmd_buf_wr(CMD_BUF_T *ptr, unsigned char Data);
static unsigned char cmd_buf_rd(CMD_BUF_T *ptr);
static void cmd_length_wr(CMD_LENGTH_T *ptr, unsigned char Data);
static unsigned char cmd_length_rd(CMD_LENGTH_T *ptr);
static unsigned char cmd_valid_check(unsigned char ModuleID);
static unsigned short cmd_crc(unsigned char *data, unsigned char length);
static unsigned short cmd_crc_uart(unsigned char length);
static void cmd_valid_response(unsigned char f_response);
static void cmd_cfg_validfb_packing(FUNC_CODE_RESPONSE_T status);
static void cmd_rb_validfb_packing(void);

static void bat_info_buf_wr(unsigned char chno);
static void bat_info_buf_rd(void);
static void bat_info_wr(void);
static unsigned short bat_info_rd(void);

CMD_T cmd ={				//cmd initialization
	.dbuf = {
		.Status.BITs.Empty = 1
	},
	.lbuf = {
		.Status.BITs.Empty = 1
	},
	.wr = cmd_wr,
	.rd = cmd_rd,
	.length_wr = cmd_length_wr,
	.length_rd = cmd_length_rd,
	.buf_wr = cmd_buf_wr,
	.buf_rd = cmd_buf_rd,
	.valid = {
		.addr = 0,
		.func = FUN01_WAIT_FOR_CMD,
		.crc = 0,
		.length = 0,
		.data = {0}
	},
	.crc = cmd_crc,
	.crc_uart = cmd_crc_uart,
	.check = cmd_valid_check,
	.response = cmd_valid_response
};
BAT_INTO_T bat_info = {		//bat_info Initial
	.buf = {
		.data = {0},
		.pWrite = {
			.Val = 0
		},
		.pRead = {
			.Val = 0
		},
		.Wused = {
			.Val = 0
		},
		.Status = {
			.BITs = {
				.Empty = 1
			}				
		}
	},
	.buf_wr = bat_info_buf_wr,
	.buf_rd = bat_info_buf_rd,
	.wr = bat_info_wr,
	.rd = bat_info_rd
};
static void cmd_wr(void){
	unsigned char temp_length = uart.receive_dbuf.Wused.Val;
	cmd_length_wr(&cmd.lbuf,temp_length);
	while(temp_length --){
		cmd_buf_wr(&cmd.dbuf,uart.receive_dbuf_rd());
	};
};
void cmd_rd(unsigned char *str, unsigned char length){			//Read from uart receive buffer and write into *str
	unsigned char cnt = 0;
	while(length -- ){
		*(str+cnt) = uart.receive_dbuf_rd();
		cnt++;
	}
};
static void cmd_buf_wr(CMD_BUF_T *ptr, unsigned char Data){
	unsigned char depth = 0;
	depth = sizeof(ptr->Data);
	if(ptr->Status.BITs.Full == 0)
	{
		ptr->Data[ptr->pWrite] = Data;
		ptr->Status.BITs.Empty = 0;
		if (ptr->pWrite == (depth-1))//                   RXFIFO_DEPTH-1)
			ptr->pWrite = 0;
		else
			ptr->pWrite ++;
		if(ptr->pRead == ptr->pWrite)
			ptr->Status.BITs.Full = 1;
		ptr->Wused++;
	}
}
static unsigned char cmd_buf_rd(CMD_BUF_T *ptr){
	unsigned char depth = 0;
	unsigned char temp = 0;
	depth = sizeof(ptr->Data);
	if(ptr->Status.BITs.Empty == 0)
	{
		temp = ptr->Data[ptr->pRead];
		ptr->Status.BITs.Full = 0;
		if(ptr->pRead == depth-1)
			ptr->pRead = 0;
		else
			ptr->pRead ++;			
		if(ptr->pRead == ptr->pWrite)
		{
			ptr->Status.BITs.Empty = 1;
		}
		ptr->Wused--;
	}
	return(temp);
}
static void cmd_length_wr(CMD_LENGTH_T *ptr, unsigned char Data){
	unsigned char depth = 0;
	depth = sizeof(ptr->Data);
	if(ptr->Status.BITs.Full == 0)
	{
		ptr->Data[ptr->pWrite] = Data;
		ptr->Status.BITs.Empty = 0;
		if (ptr->pWrite == (depth-1))//                   RXFIFO_DEPTH-1)
			ptr->pWrite = 0;
		else
			ptr->pWrite ++;
		if(ptr->pRead == ptr->pWrite)
			ptr->Status.BITs.Full = 1;
		ptr->Wused++;
	}
}
static unsigned char cmd_length_rd(CMD_LENGTH_T *ptr){
	unsigned char depth = 0;
	unsigned char temp = 0;
	depth = sizeof(ptr->Data);
	if(ptr->Status.BITs.Empty == 0)
	{
		temp = ptr->Data[ptr->pRead];
		ptr->Status.BITs.Full = 0;
		if(ptr->pRead == depth-1)
			ptr->pRead = 0;
		else
			ptr->pRead ++;			
		if(ptr->pRead == ptr->pWrite)
		{
			ptr->Status.BITs.Empty = 1;
		}
		ptr->Wused--;
	}
	return(temp);
}
static unsigned short cmd_crc(unsigned char *data, unsigned char length){
	unsigned short CRC_REG = 0xFFFF;//Step 1
	unsigned short shift = 0;
	while(length--)
	{	
		CRC_REG ^= *data;		//Step 2
		for(shift = 0;shift<8;shift++)
		{
			if(CRC_REG&0x0001)
			{
				CRC_REG = CRC_REG >> 1;			//Step 3
				CRC_REG ^= 0xA001;
			}
			else
			{
				CRC_REG = CRC_REG >> 1;			//Step 3
			}
		}
		data++;
	}
	return(CRC_REG);
}
static unsigned short cmd_crc_uart(unsigned char length){
	unsigned short CRC_REG = 0xFFFF;//Step 1
	unsigned short shift = 0;	
	RECEIVE_DBUF_PTR_T pread;
	pread.Val = uart.receive_dbuf.pRead.Val;
	while(length--)
	{	
		CRC_REG ^= uart.receive_dbuf.Data[pread.Val];								//*data;		//Step 2
		for(shift = 0;shift<8;shift++)
		{
			if(CRC_REG&0x0001)
			{
				CRC_REG = CRC_REG >> 1;			//Step 3
				CRC_REG ^= 0xA001;
			}
			else
			{
				CRC_REG = CRC_REG >> 1;			//Step 3
			}
		}
		pread.Val++;
	}
	return(CRC_REG);	
}
static unsigned char cmd_valid_check(unsigned char ModuleID){
	unsigned char length = 0;
	unsigned char addr = 0;
	unsigned short crc_cal = 0;
	unsigned short crc_cmd = 0;
	RECEIVE_DBUF_PTR_T phead;
	RECEIVE_DBUF_PTR_T ptr;
	phead.Val = uart.receive_dbuf.pRead.Val;
	if(uart.receive_lbuf.Status.BITs.Empty == 1){				//NO CMD
		cmd.valid.func = FUN01_WAIT_FOR_CMD;
		return(0);																//No Response Needed.
	}
	else{
		addr = uart.receive_dbuf.Data[phead.Val];			//1st Byte should be the Address; Update the CMD Length	
		if((addr != ModuleID)&&(addr != 0)){										//addr Mismatch, Read out the rest of bytes and do nothing.
			length = uart.receive_lbuf_rd();
			cmd.valid.func = ERR03_CMD_ADDR_MISMATCH;
			while(length--){													//Remove all the rest bytes of no interest. 
				uart.receive_dbuf_rd();
			}
			return(0);																//No Response Needed.		
		}
		else {																			//addr Matching or addr = 0!! Response Needed
			if(cmd.f_valid == 0){											//No Valid Right now. or All the previous cmd has been responsed.
				length = uart.receive_lbuf_rd();
				if(length < 5){														//Check the CMD Length, at least, there should be addr func, crc(2 byte)
					cmd.valid.func = ERR01_CMD_TOO_SHORT;	
					return(0);//Wait for the current cmd to be responsed.					
				}
				else if (length > CMD_DATA_LENGTH_MAX){		//CMD Length need to be shorter the depth of the CMD buffer.
					cmd.valid.func = ERR02_CMD_TOO_LONG;
					return(0);//Wait for the current cmd to be responsed.					
				}
				else{																			//Length is OK, check the CRC Code.
					ptr.Val = phead.Val+length-1;
					crc_cmd = (uart.receive_dbuf.Data[ptr.Val]<<8);
					ptr.Val = phead.Val+length-2;
					crc_cmd = crc_cmd + uart.receive_dbuf.Data[ptr.Val];
					//crc_cmd = ((uart.receive_dbuf.Data[uart.receive_dbuf.pRead.Val+length-2]<<8)+uart.receive_dbuf.Data[uart.receive_dbuf.pRead.Val+length-1]);
					crc_cal = cmd_crc_uart(length-2);
					if(crc_cal == crc_cmd){									//CRC matched, CMD Valid!!
						//Unpack the CMD data and store it into valid buffer.
						cmd.valid.addr = uart.receive_dbuf_rd();
						cmd.valid.func = (FUNC_CODE_T)uart.receive_dbuf_rd();
						cmd.rd(cmd.valid.data.array,(length-4));		//read all the rest bytes except addr, func, crc(2bytes)
						cmd.valid.crc = uart.receive_dbuf_rd();
						cmd.valid.crc = ((cmd.valid.crc<<8) + uart.receive_dbuf_rd());
						cmd.valid.length = length-4;
						cmd.f_valid = 1;
						return(1);//Response Needed.
					}
					else{																		//CRC Mismatch, CMD CRC Error
						cmd.valid.func = ERR04_CMD_CRC_MISMATCH;
						while(length--){													//Remove all the rest bytes of no interest. 
							uart.receive_dbuf_rd();
						}
						return(0);//Wait for the current cmd to be responsed.
					}
				}
			}
			else{
				return(0);//Wait for the current cmd to be responsed.
			}
		}
	}
}

static void cmd_cfg_validfb_packing(FUNC_CODE_RESPONSE_T status){
	unsigned short crc_temp = 0;
	unsigned char byte_num = 0;
	SENDDDD_DBUF_PTR_T phead;
	phead.Val = uart.send_dbuf.pWrite.Val;
	uart.send_dbuf_wr(cmd.valid.addr);
	uart.send_dbuf_wr(cmd.valid.func);
	uart.send_dbuf_wr((unsigned char)status);
	byte_num = 3;
	crc_temp = uart.send_crc(phead,byte_num);
	uart.send_dbuf_wr(((crc_temp&0xFF00)>>8));
	uart.send_dbuf_wr((crc_temp&0x00FF));
	byte_num = 5;
	uart.send_lbuf_wr(byte_num);
}

static void cmd_rb_validfb_packing(void){
	unsigned char chno = 0;
	SENDDDD_DBUF_PTR_T phead;
	unsigned short crc_temp = 0;
	unsigned short byte_num = 0;
	phead.Val = uart.send_dbuf.pWrite.Val;
	uart.send_dbuf_wr(cmd.valid.addr);
	uart.send_dbuf_wr(cmd.valid.func);
	byte_num = 2;
	switch(cmd.valid.func){
		case FUN64_INFORB_CN0352:{
			uart.send_dbuf_wr(((cn0352.name&0xFF00)>>8));
			uart.send_dbuf_wr((cn0352.name&0x00FF));
			uart.send_dbuf_wr(((cn0352.FirmVer&0xFF00)>>8));
			uart.send_dbuf_wr((cn0352.FirmVer&0x00FF));
			uart.send_dbuf_wr(cn0352.ModuleID);
			uart.send_dbuf_wr(cn0352.validch.qty);
			uart.send_dbuf_wr(cn0352.validch.flag.val);
			uart.send_dbuf_wr(CONST_DWORD_ALIGNMENT);
//			for(chno = 0; chno < cn0352.validch.qty; chno ++){
//				uart.send_dbuf_wr(cn0352.validch.index[chno]);
//			}
			uart.send_dbuf_wr(*((unsigned char*)&cn0352.samplerate + 3));
			uart.send_dbuf_wr(*((unsigned char*)&cn0352.samplerate + 2));
			uart.send_dbuf_wr(*((unsigned char*)&cn0352.samplerate + 1));
			uart.send_dbuf_wr(*((unsigned char*)&cn0352.samplerate + 0));
			byte_num = 2+12;
//			byte_num = 2+11+cn0352.validch.qty;
			break;
		}
		case FUN65_INFORB_BAT:{
			byte_num += bat_info.rd();
			break;
		}
		case FUN66_INFORB_CHG:{
			for(chno = 0; chno < 8 ; chno ++){
				if(cn0352.ch[chno].active == 1){
					uart.send_dbuf_wr(chno);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.IntActSta.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.ChargerSta1.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.ChargerSta2.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.FaultReg.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.BatShort.Val);
					byte_num += 6;
				}
			}
			break;
		}
		case FUN67_INFORB_ALL:{
			for(chno = 0; chno < 8 ; chno ++){
				if(cn0352.ch[chno].active == 1){
					uart.send_dbuf_wr(chno);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.IntActSta.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.ChargerSta1.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.ChargerSta2.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.FaultReg.Val);
					uart.send_dbuf_wr(cn0352.ch[chno].adp5065r.reg.BatShort.Val);
					byte_num += 6;
				}
			}
			byte_num += bat_info.rd();
			break;
		}
		default:
			break;
	}
	crc_temp = uart.send_crc(phead,byte_num);
	uart.send_dbuf_wr(((crc_temp&0xFF00)>>8)); byte_num++;
	uart.send_dbuf_wr((crc_temp&0x00FF)); byte_num++;
	uart.send_lbuf_wr(byte_num);
};
static void cmd_validfb_send(void){
	while(1){if(uart.status == UART_IDLE) break;}
	uart.modbusend_string_int();
}
static void cmd_valid_response(unsigned char f_response){
	if(f_response){
		if(cmd.valid.addr == 0x00){									//Broadcast Function
			switch(cmd.valid.func){
				case FUN40_CFG_MODULEID:{
					if((cmd.valid.data.Val_FUN40.key[0] == 0x0F)&&
						 (cmd.valid.data.Val_FUN40.key[1] == 0xF0)&&
					   (cmd.valid.data.Val_FUN40.key[2] == 0x55)&&
					   (cmd.valid.data.Val_FUN40.key[3] == 0xAA)){
						cn0352.ModuleID = cmd.valid.data.Val_FUN40.name;
						if(cn0352.run.cfg.moduleIDupd()){
							cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);							 
						}
						else {
							cmd_cfg_validfb_packing(FUNC_RESPONSE_FAILED);
						}
						cmd_validfb_send();
						break;
					}
				}
				default:{
					//cmd_cfg_validfb_packing(FUNC_RESPONSE_FUN_UNKNOW);
					break;
				}
			}
		}
		else{																				//Deducated Function
			if(cmd.valid.func & 0x80){									//Error
				switch(cmd.valid.func){
					case ERR01_CMD_TOO_SHORT:{
						cmd_cfg_validfb_packing(FUNC_RESPONSE_FAILED);
						cmd_validfb_send();
						break;
					}
					case ERR02_CMD_TOO_LONG:{
						cmd_cfg_validfb_packing(FUNC_RESPONSE_FAILED);
						cmd_validfb_send();
						break;
					}
					case ERR03_CMD_ADDR_MISMATCH:{
						//No Response
						break;
					}	
					case ERR04_CMD_CRC_MISMATCH:{
						cmd_cfg_validfb_packing(FUNC_RESPONSE_FAILED);
						cmd_validfb_send();
						break;
					}
					default:{
						break;
					}	
				}
			}
			else{																				//Valid Function & addr != 0
				switch(cmd.valid.func){
					case FUN01_WAIT_FOR_CMD:{
						break;
					}
					case FUN02_VALIDCH_SCAN:{
						cn0352.run.rb.validchsacn();
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}
					case FUN03_MODULE_RESET:{
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						while(1){if(uart.status == UART_IDLE) break;}		//Waiting for Sending Complete and Reset the Module
						cn0352.run.cfg.sw_reset();
						break;
					}
					case FUN04_IN_CUR_LIMIT:{
						cn0352.run.cfg.InCurrLimit_Allset(cmd.valid.data.array[0]);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}
					case FUN05_TC_CUR_THOLD:{
						cn0352.run.cfg.TrickleCurr_Allset(cmd.valid.data.array[0]);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}
					case FUN06_CC_CUR_VALUE:{
						cn0352.run.cfg.FastCurr_Allset(cmd.valid.data.array[0]);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}
					case FUN07_CV_VOLT_TERM:{
						cn0352.run.cfg.TermVolt_Allset(cmd.valid.data.array[0]);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}			
					case FUN08_CV_CURR_TERM:{
						cn0352.run.cfg.TermCurr_Allset(cmd.valid.data.array[0]);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}	
					case FUN09_DC_CURR:{
						cn0352.run.cfg.DisTermCurr_Allset(cmd.valid.data.array);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}	
					case FUN10_DC_VOLT:{
						cn0352.run.cfg.DisTermVolt_Allset(cmd.valid.data.array);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}
					case FUN16_TEST_CYCLE:{
						cn0352.run.cfg.test_cycle(cmd.valid.data.array[0]);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						cmd_validfb_send();
						break;
					}	
					case FUN11_CONFIG_ALL:{
						cn0352.run.cfg.CHG_CFG_ALL((FORMAT_CFG_ALL_T *)cmd.valid.data.array);
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);			
						cmd_validfb_send();
						break;
					}
					case FUN20_TEST_START:{
						cn0352.test_cycle_cnt = 0;
						cn0352.run.test.start_all();
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);			
						cmd_validfb_send();						
						break;
					}
					case FUN21_TEST_STOP:{
						cn0352.test_cycle_cnt = 0;
						cn0352.run.test.stop_all();
						cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);			
						cmd_validfb_send();						
						break;
					}					
					case FUN32_CFG_BAUDRATE:{
						if (cmd.valid.data.array[0] < UART_BAUDRATE_VALID_NUM){
							cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
							cmd_validfb_send();
							while(1){if(uart.status == UART_IDLE) break;}		//Waiting for Sending Complete and Reset the Module
							cn0352.baudrate = uart.BAUDRATE_VALID[cmd.valid.data.array[0]];
							cn0352.run.cfg.baudrateupd();
						}
						else{
							cmd_cfg_validfb_packing(FUNC_RESPONSE_BAUDRATE_OUT_RANGE);
							cmd_validfb_send();
						}
						break;
					}
					case FUN33_CFG_FREQ_ADC:{
						if(cmd.valid.data.array[0] < ADC_UPDATE_VALID_NUM){
							cn0352.adc_odr_index = cmd.valid.data.array[0];
							cn0352.run.cfg.init();
							cmd_cfg_validfb_packing(FUNC_RESPONSE_SUCCEED);
						}
						else{
							cmd_cfg_validfb_packing(FUNC_RESPONSE_SAMPLERATE_OUT_RANGE);
						}
						cmd_validfb_send();	
						break;
					}
					case FUN64_INFORB_CN0352:{
						cmd_rb_validfb_packing();
						cmd_validfb_send();
						break;
					}
					case FUN65_INFORB_BAT:{	
						if((bat_info.buf.Status.BITs.Empty == 0)&&(bat_info.buf.Wused.Val > cn0352.validch.qty)){//if there are some channels with data updated
							cmd_rb_validfb_packing();
							cmd_validfb_send();
						}
//						else{
//							cmd_cfg_validfb_packing(FUNC_RESPONSE_NO_UPDATED_DATA);
//						}
						break;
					}
					case FUN66_INFORB_CHG:{
						cmd_rb_validfb_packing();
						cmd_validfb_send();
						break;
					}
					case FUN67_INFORB_ALL:{
						cmd_rb_validfb_packing();
						cmd_validfb_send();
						break;
					}						
					default:{
						break;
					}
				}
			}
		}
		cmd.f_valid = 0;
	}
};

static void bat_info_buf_wr(unsigned char chno){
	if(bat_info.buf.Status.BITs.Full == 0){			//Buffer Not Full
		{																							//Write New Data into Bat Info Buffer
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.chno = chno;
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.status = cn0352.ch[chno].status;
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.mstatus = cn0352.ch[chno].mstatus.Val;
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.count = cn0352.ch[chno].count;
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.cycle_cnt = cn0352.ch[chno].test_cycle_cnt;
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.volt = cn0352.ch[chno].bat.volt;
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.curr = cn0352.ch[chno].bat.curr;
			bat_info.buf.data[bat_info.buf.pWrite.Val].reg.temp = cn0352.ch[chno].bat.temp;			
		}
		{																							//Update Status
			bat_info.buf.Status.BITs.Empty = 0;					
			bat_info.buf.pWrite.Val++;
			if(bat_info.buf.pWrite.Val == bat_info.buf.pRead.Val){
				bat_info.buf.Status.BITs.Full = 1;
			}
			bat_info.buf.Wused.Val++;
		}
	}
};
static void bat_info_buf_rd(void){						//Read out new data from bat info buffer and write into UART Send Buffer for sending to PC.
	unsigned char *p;
	if(bat_info.buf.Status.BITs.Empty == 0){		//bat info buffer not empt
		{																							//Read out new data from bat info buffer and write into UART Send Buffer for sending to PC.
			uart.send_dbuf_wr(bat_info.buf.data[bat_info.buf.pRead.Val].reg.chno); 							
			uart.send_dbuf_wr(bat_info.buf.data[bat_info.buf.pRead.Val].reg.status);	
			p = (unsigned char *)&bat_info.buf.data[bat_info.buf.pRead.Val].reg.mstatus;
			uart.send_dbuf_wr(*(p+1)); uart.send_dbuf_wr(*p); 			
			p = (unsigned char *)&bat_info.buf.data[bat_info.buf.pRead.Val].reg.count;
			uart.send_dbuf_wr(*(p+1)); uart.send_dbuf_wr(*p); 
			p = (unsigned char *)&bat_info.buf.data[bat_info.buf.pRead.Val].reg.cycle_cnt;
			uart.send_dbuf_wr(*(p+1)); uart.send_dbuf_wr(*p); 
			p = (unsigned char *)&bat_info.buf.data[bat_info.buf.pRead.Val].reg.volt;
			uart.send_dbuf_wr(*(p+3)); uart.send_dbuf_wr(*(p+2));uart.send_dbuf_wr(*(p+1));uart.send_dbuf_wr(*p);
			p = (unsigned char *)&bat_info.buf.data[bat_info.buf.pRead.Val].reg.curr;
			uart.send_dbuf_wr(*(p+3)); uart.send_dbuf_wr(*(p+2));uart.send_dbuf_wr(*(p+1));uart.send_dbuf_wr(*p);
			p = (unsigned char *)&bat_info.buf.data[bat_info.buf.pRead.Val].reg.temp;
			uart.send_dbuf_wr(*(p+3)); uart.send_dbuf_wr(*(p+2));uart.send_dbuf_wr(*(p+1));uart.send_dbuf_wr(*p);
			//uart.send_dbuf_wr(*p); uart.send_dbuf_wr(*(p+1)); uart.send_dbuf_wr(*(p+2)); uart.send_dbuf_wr(*(p+3));
			//p = (unsigned char *)&bat_info.buf.data[bat_info.buf.pRead.Val].reg.vdisrb;			
			//uart.send_dbuf_wr(*p); uart.send_dbuf_wr(*(p+1)); uart.send_dbuf_wr(*(p+2)); uart.send_dbuf_wr(*(p+3));
		}
		{																							//Update Status
			bat_info.buf.Status.BITs.Full = 0;
			bat_info.buf.pRead.Val++;
			if(bat_info.buf.pWrite.Val == bat_info.buf.pRead.Val){
				bat_info.buf.Status.BITs.Empty = 1;
			}
			bat_info.buf.Wused.Val--;
		}
	}
};
static void bat_info_wr(void){
	unsigned char chno = 0;
	if(bat_info.buf.Status.BITs.Full == 0){
	//if((bat_info.buf.Wused.Val + cn0352.validch.qty) < BAT_INFO_BUF_LENGTH){
		for(chno = 0; chno < 8; chno++){
			if(cn0352.ch[chno].bat.f_upd == 1){
				bat_info_buf_wr(chno);
				cn0352.ch[chno].bat.f_upd = 0;
				break;
			}
		}
	}
}
static unsigned short bat_info_rd(void){
	unsigned short byte_num = 0;
	unsigned chno = 0;
	while(
		(((unsigned long)uart.send_dbuf.Wused.Val + (unsigned long)cn0352.validch.qty*(unsigned long)LENGTH_OF_BAT_INFO_FLO)<UART_SENDDDD_DBUF_SIZE)&&
		(bat_info.buf.Wused.Val > cn0352.validch.qty)){		//Uart Send Buffer has enough space for writing.
		for(chno = 0; chno < cn0352.validch.qty; chno++){
			bat_info_buf_rd();
			byte_num += LENGTH_OF_BAT_INFO_FLO;
		}
	}
	return(byte_num);
}

