
/*
*===========================================================
*        File Name: cmd.h
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/
#ifndef CMD_H_
#define CMD_H_

#include "inc\ADuCM360.h"
#include "inc\system_ADUCM360.h"

#define CMD_DATA_BUF_SIZE						64
#define CMD_LENGTH_BUF_SIZE					8
#define CMD_DATA_LENGTH_MAX					64
#define CMD_FB_DATA_LENGTH_MAX			2
#define KEY_LENGTH_FUN40_CFG_MODULEID	4
#define LENGTH_OF_BAT_INFO_FLO 			20//sizeof(BAT_INFO_FLO_BUF_S_T)

typedef enum {
	FUNC_RESPONSE_SUCCEED = 0x55,
	FUNC_RESPONSE_FAILED	= 0xAA,
	FUNC_RESPONSE_BAUDRATE_OUT_RANGE = 0xAB,
	FUNC_RESPONSE_NO_UPDATED_DATA = 0xAC,
	FUNC_RESPONSE_MODULEID_KEY_ERROR = 0xAD,
	FUNC_RESPONSE_MODULEID_FLASHERR = 0xAE,
	FUNC_RESPONSE_FUN_UNKNOW = 0xAF,
	FUNC_RESPONSE_SAMPLERATE_OUT_RANGE = 0xB0
}FUNC_CODE_RESPONSE_T;

typedef struct{	//CMD_BUF_T
		unsigned char Data[CMD_DATA_BUF_SIZE];
		unsigned int pWrite;
		unsigned int pRead;
		unsigned int Wused;
//		unsigned int pSRead;
		union
		{
			unsigned char Val;
			struct 
			{
				__IO uint8_t	Full  : 1;
				__IO uint8_t	Empty : 1;
			} BITs;
		}Status;
}CMD_BUF_T;
typedef struct{	//CMD_LENGTH_T
		unsigned char Data[CMD_LENGTH_BUF_SIZE];
		unsigned int pWrite;
		unsigned int pRead;
		unsigned int Wused;
//		unsigned int pSRead;
		union
		{
			unsigned char Val;
			struct 
			{
				__IO uint8_t	Full  : 1;
				__IO uint8_t	Empty : 1;
			} BITs;
		}Status;
}CMD_LENGTH_T;

typedef enum  {	//FUNC_CODE_T
//Define Function Code			All Function Code would be 0b0XXX XXXX
	//Configuration Function (Writing Function)
		FUN01_WAIT_FOR_CMD	= 0x00,			//Idle Status, Waiting for Valid Command from PC
		FUN02_VALIDCH_SCAN	= 0x01,			//Talk to All the Slot to find Valid IO Board.
		FUN03_MODULE_RESET	= 0x02,			//Reset the CN0352 Module and ALL IO									Update All Valid Channels
		FUN04_IN_CUR_LIMIT	= 0x03,			//Charging Input Current Limitation										Update All Valid Channels
		FUN05_TC_CUR_THOLD	= 0x04,			//Trickle Charging Current Threshold									Update All Valid Channels
		FUN06_CC_CUR_VALUE	= 0x05,			//Fast Constant-Current Charge Current.								Update All Valid Channels
		FUN07_CV_VOLT_TERM	= 0x06,			//Fast Constant-Vlotage Charge Termination Voltage		Update All Valid Channels
		FUN08_CV_CURR_TERM	= 0x07,			//Fast Constant-Voltage Charge Termination Current		Update All Valid Channels
		FUN09_DC_CURR				=	0x08,			//Discharge Step1 Current															Update All Valid Channels
		FUN10_DC_VOLT				=	0x09,			//Discharge Step1 Termination Voltage									Update All Valid Channels
		FUN11_CONFIG_ALL		= 0x0A,			//Update ALL Configuration														Update All Valid Channels
		//FUN12_DC_CURR_2			=	0x0B,			//Discharge Step2 Current															Update All Valid Channels
		//FUN13_DC_VOLT_2			=	0x0C,			//Discharge Step2 Termination Voltage									Update All Valid Channels
		//FUN14_DC_CURR_3			=	0x0D,			//Discharge Step3 Current															Update All Valid Channels
		//FUN15_DC_VOLT_3			=	0x0E,			//Discharge Step3 Termination Voltage									Update All Valid Channels
		FUN16_TEST_CYCLE		=	0x0F, 		//Charging-Discharging Cycle Number for Testing
		FUN20_TEST_START		= 0x14,			//Started Testing
		FUN21_TEST_STOP			= 0x15,			//Stop Testing
	  //ADuCM360 Function
		FUN32_CFG_BAUDRATE  = 0x20,			//Config Uart Baudrate
		FUN33_CFG_FREQ_ADC	= 0x21,			//Modify ADC Sample Frequency
	  FUN40_CFG_MODULEID	= 0x28,			//Modify the Module ID
  //Reading Back Functions (Reading Function)
		FUN64_INFORB_CN0352 = 0x40,			//Read CN0352 Module Information
		FUN65_INFORB_BAT		= 0x41,			//Read All the information data for battery in BAT Info Buffer
		FUN66_INFORB_CHG		= 0x42,			//Read Charger Status
		FUN67_INFORB_ALL		= 0x43,			//Read BAT_INFO + CHG_STATUS

	//Define Error								All    Error Code would be 0b1XXX XXXX
		ERR01_CMD_TOO_SHORT = 0x80,
		ERR02_CMD_TOO_LONG	= 0x81,
		ERR03_CMD_ADDR_MISMATCH	= 0x82,
		ERR04_CMD_CRC_MISMATCH	= 0x83
} FUNC_CODE_T;
typedef struct {
	unsigned char key[KEY_LENGTH_FUN40_CFG_MODULEID];
	unsigned char name;
} CMD_FRAME_FUN40_CFG_MODULEID_T;
typedef struct{	//CMD_FRAME_T
	unsigned char addr;
	FUNC_CODE_T func;
	unsigned short crc;
	unsigned short length;
	union {
		unsigned char array[CMD_DATA_LENGTH_MAX];
		CMD_FRAME_FUN40_CFG_MODULEID_T Val_FUN40;
	}data;
}CMD_FRAME_T;
typedef struct{	//CMD_T
	CMD_BUF_T dbuf;
	CMD_LENGTH_T lbuf;
	void (*buf_wr)(CMD_BUF_T *ptr, unsigned char Data);
	unsigned char (*buf_rd)(CMD_BUF_T *ptr);
	void (*length_wr)(CMD_LENGTH_T *ptr, unsigned char Data);
	unsigned char (*length_rd)(CMD_LENGTH_T *ptr);	
	void(*wr)(void);
	void(*rd)(unsigned char *str, unsigned char length);	
	
	CMD_FRAME_T valid;
	unsigned char valid_fb[CMD_FB_DATA_LENGTH_MAX];
	unsigned char valid_fb_length;
	unsigned char f_valid;
	unsigned char (*check)(unsigned char);
	unsigned short (*crc)(unsigned char *data, unsigned char length);
	unsigned short (*crc_uart)(unsigned char length);
	void (*response)(unsigned char f_response);
} CMD_T;
extern CMD_T cmd;

typedef struct{	//FORMAT_CFG_InCurrLimit_T
	unsigned char data;
}FORMAT_CFG_InCurrLimit_T;
typedef struct{	//FORMAT_CFG_TrickleVth_T
	unsigned char data;
}FORMAT_CFG_TrickleVth_T;
typedef struct{	//FORMAT_CFG_TrickleCurr_T
	unsigned char data;
}FORMAT_CFG_TrickleCurr_T;
typedef struct{	//FORMAT_CFG_FastCurr_T
	unsigned char data;
}FORMAT_CFG_FastCurr_T;
typedef struct{	//FORMAT_CFG_TermVolt_T
	unsigned char data;
}FORMAT_CFG_TermVolt_T;
typedef struct{	//FORMAT_CFG_TermCurr_T
	unsigned char data;
}FORMAT_CFG_TermCurr_T;
typedef struct{	//FORMAT_CFG_DisTermVolt_T
	unsigned char data[3];
}FORMAT_CFG_DisTermVolt_T;
typedef struct{	//FORMAT_CFG_DisTermCurr_T
	unsigned char data[3];
}FORMAT_CFG_DisTermCurr_T;
typedef struct{	//FORMAT_CFG_TestCycle_T
	unsigned short data;
}FORMAT_CFG_TestCycle_T;
typedef union {	//FORMAT_CFG_GENERAL_U_T
	FORMAT_CFG_InCurrLimit_T InCurrLimit;
	FORMAT_CFG_TrickleVth_T  TrickleVth;
	FORMAT_CFG_TrickleCurr_T TrickleCurr;
	FORMAT_CFG_FastCurr_T    FastCurr;
	FORMAT_CFG_TermVolt_T    TermVolt;
	FORMAT_CFG_TermCurr_T    TermCurr;
	FORMAT_CFG_DisTermVolt_T DisTermVolt;
	FORMAT_CFG_DisTermCurr_T DisTermCurr;
	FORMAT_CFG_TestCycle_T   TestCycle;
}FORMAT_CFG_GENERAL_U_T;
typedef struct{	//FORMAT_CFG_ALL_S_T
	FORMAT_CFG_InCurrLimit_T InCurrLimit;
	FORMAT_CFG_TrickleVth_T  TrickleVth;
	FORMAT_CFG_TrickleCurr_T TrickleCurr;
	FORMAT_CFG_FastCurr_T    FastCurr;
	FORMAT_CFG_TermVolt_T    TermVolt;
	FORMAT_CFG_TermCurr_T    TermCurr;
	FORMAT_CFG_DisTermVolt_T DisTermVolt;
	FORMAT_CFG_DisTermCurr_T DisTermCurr;
	FORMAT_CFG_TestCycle_T   TestCycle;
}FORMAT_CFG_ALL_S_T;
typedef union {	//FORMAT_CFG_GENERAL_T @
	FORMAT_CFG_GENERAL_U_T REG;
	unsigned char ARRAY[sizeof(FORMAT_CFG_GENERAL_U_T)];
}FORMAT_CFG_GENERAL_T;
typedef union {	//FORMAT_CFG_ALL_T @
	FORMAT_CFG_ALL_S_T REG;
	unsigned char ARRAY[sizeof(FORMAT_CFG_ALL_S_T)];
}FORMAT_CFG_ALL_T;

#define BAT_INFO_BUS_WIDTH		5			//Need to satified with the equisition: 2^UART_SENDDDD_DBUF_BUS_WIDTH >= 2^BAT_INFO_BUS_WIDTH*20
#define BAT_INFO_BUF_LENGTH		(1<<BAT_INFO_BUS_WIDTH)


typedef struct{				//BAT_INFO_INT_BUF_S_T
		uint8_t chno;		
	  uint8_t status;
	  uint16_t mstatus;
		uint16_t count;
		uint16_t cycle_cnt;
		uint32_t raw_volt;
		uint32_t raw_curr;
		uint32_t raw_temp;
		uint32_t raw_vdisrb;
}BAT_INFO_INT_BUF_S_T;
typedef struct{				//BAT_INFO_FLO_BUF_S_T
		uint8_t chno;		
		uint8_t status;
		uint16_t mstatus;
		uint16_t count;
		uint16_t cycle_cnt;
		float volt;		//Store the 4byte value converted from real voltage
		float curr;
		float temp;
}BAT_INFO_FLO_BUF_S_T;

typedef union {				//BAT_INFO_BUF_U_T
	BAT_INFO_FLO_BUF_S_T reg;
	uint8_t array[sizeof(BAT_INFO_FLO_BUF_S_T)];
}BAT_INFO_BUF_U_T;
typedef struct{				//BAT_INTO_BUF_T
	BAT_INFO_BUF_U_T data[BAT_INFO_BUF_LENGTH];
	struct {		//pWrite
		uint8_t Val : BAT_INFO_BUS_WIDTH;
		uint8_t Reserve : (8-BAT_INFO_BUS_WIDTH);
	}pWrite;
	struct {		//pRead
		uint8_t Val : BAT_INFO_BUS_WIDTH;
		uint8_t Reserve : (8-BAT_INFO_BUS_WIDTH);
	}pRead;
	struct {		//Wused
		uint8_t Val : BAT_INFO_BUS_WIDTH+1;
		uint8_t Reserve : (8-BAT_INFO_BUS_WIDTH-1);  
	}Wused;	
	union  {		//Status
		unsigned char Val;
		struct 
		{
			__IO uint8_t	Full  : 1;
			__IO uint8_t	Empty : 1;
		} BITs;
	}Status;
}BAT_INTO_BUF_T;
typedef struct{				//BAT_INTO_T
	BAT_INTO_BUF_T buf;
	void(*buf_wr)(unsigned char chno);
	void(*buf_rd)(void);
	void(*wr)(void);
	unsigned short (*rd)(void);
}BAT_INTO_T;
extern BAT_INTO_T bat_info;


#endif /*CMD_H_*/

