
/*
*===========================================================
*        File Name: timer1.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "timer1.h"
#include "hardware.h"
#include "uart.h"
#include "cmd.h"

static void timer1_timeout(int cnt);
static void timer1_init(void);
static void timer1_enable(void);
static void timer1_disable(void);
static void timer1_clear_int(void);
static void timer1_recount(void);
extern void GP_Tmr1_Int_Handler(void);

TIMER1_T timer1 = {
	.cnt = 0,
	.timeout = timer1_timeout,
	.init = timer1_init,
	.enable = timer1_enable,
	.disable = timer1_disable,
	.clear_int = timer1_clear_int,
	.recount = timer1_recount,
	.Int_Handler = GP_Tmr1_Int_Handler
};
static void timer1_timeout(int cnt){
	pADI_TM1->LD = cnt;
};
static void timer1_init(void){
	timer1.cnt_chartimex3P5 = (unsigned int)(TIMER1_CLK_FREQUENCY*uart.chartimex3P5);
	timer1.cnt_chartimex4   = (unsigned int)(TIMER1_CLK_FREQUENCY*uart.chartimex4);
	GptCfg(pADI_TM1,TCON_CLK_PCLK,TCON_PRE_DIV16,TCON_MOD_PERIODIC|TCON_RLD|TCON_EVENTEN_DIS);
	pADI_TM1->LD = timer1.cnt_chartimex3P5;
}
static void timer1_enable(void){
	pADI_TM1->CON |= (unsigned short)TCON_ENABLE_MSK ;// Enable Timer
}
static void timer1_disable(void){
	pADI_TM1->CON &= (unsigned short)(~TCON_ENABLE_MSK) ;// Disable Timer
}
static void timer1_clear_int(void){
	pADI_TM1->CLRI = TSTA_TMOUT;	//Clear TimerOut Interrupt
}
static void timer1_recount(void){
	pADI_TM1->CON &= (unsigned short)(~TCON_ENABLE_MSK) ;// Disable Timer
	pADI_TM1->CLRI = TSTA_TMOUT;	//Clear TimerOut Interrupt
	pADI_TM1->CON |= (unsigned short)TCON_ENABLE_MSK ;// Enable Timer
}
extern void GP_Tmr1_Int_Handler(void){
	switch(uart.status)
	{
		case UART_IDLE:
		{
			timer1.disable();
			timer1.clear_int();
			break;
		}
		case UART_MODBUS_RECEIVE:		//3.5 Character Timer Out, 1 Frame Complete.
		{
			uart.status = UART_IDLE;
			timer1.disable();
			timer1.clear_int();
			// Copy All the data in uart receive buffer into CMD buffer;
			//cmd.wr();
			uart.receive_lbuf_wr(uart.receive_dbuf.Wused.Val);	//Record the Length of the Frame.
			break;
		}
		case UART_MODBUS_1stSILENCE:	 //4 Character Time to be silence.
		{
			timer1.disable();
			timer1.clear_int();
			//uart.status = UART_SENT_DATA;
			timer1.timeout(timer1.cnt_chartimex3P5);
			uart.status = UART_MODBUS_DATA;
			uart.send_string_int();
			timer1.enable();
			break;
		}
		case UART_MODBUS_2ndSILENCE: 	//4 Character Time to be silence.
		{
			timer1.disable();
			timer1.clear_int();
			uart.send_lbuf_rd();				//Remove the uart send length buffer data.
			uart.status = UART_IDLE;
			break;
		}
		default:
		{
			timer1.disable();
			timer1.clear_int();
			break;
		}
	}
}

