
/*
*===========================================================
*        File Name: uart.h
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/
#ifndef UART_H_
#define UART_H_

#include "inc\ADuCM360.h"
#include "inc\system_ADUCM360.h"
#include "common\UrtLib.h"
#include "timer1.h"
#include "common\DmaLib.h"
//#include "math.h"

#define UART_RECEIVE_DBUF_BUS_WIDTH 6
#define UART_RECEIVE_DBUF_SIZE (1<<UART_RECEIVE_DBUF_BUS_WIDTH)
#define UART_RECEIVE_LBUF_BUS_WIDTH 2
#define UART_RECEIVE_LBUF_SIZE (1<<UART_RECEIVE_LBUF_BUS_WIDTH)
#define UART_SENDDDD_DBUF_BUS_WIDTH 10
#define UART_SENDDDD_DBUF_SIZE (1<<UART_SENDDDD_DBUF_BUS_WIDTH)
#define UART_SENDDDD_LBUF_BUS_WIDTH 2
#define UART_SENDDDD_LBUF_SIZE (1<<UART_SENDDDD_LBUF_BUS_WIDTH)

#define UART_BAUDRATE_INIT			115200
#define UART_BAUDRATE_VALID_NUM 		 5

#define __RS485_EN__
#define RS485_SEND						0x01
#define RS485_RECEIVE					0x00

extern unsigned char szTemp[64] ;                       // Used to store string before printing to UART
extern unsigned char szPacketIn[32];	                       // Used to store received bytes from UART

typedef union  {		//COMLCR_T
	unsigned char Val;
	struct {
		unsigned char WLS			: 2;
		unsigned char STOP		: 1;
		unsigned char PEN			: 1;
		unsigned char EPS			: 1;
		unsigned char SP			: 1;
		unsigned char BRK			: 1;
		unsigned char Reserved: 1;
	} BITs;
} COMLCR_T;
//SENDDDD_DBUF
typedef struct {		//SENDDDD_DBUF_PTR_T
	uint16_t Val : UART_SENDDDD_DBUF_BUS_WIDTH;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_SENDDDD_DBUF_BUS_WIDTH);	
}SENDDDD_DBUF_PTR_T;
typedef struct {		//SENDDDD_DBUF_WUSED_T
	uint16_t Val : UART_SENDDDD_DBUF_BUS_WIDTH+1;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_SENDDDD_DBUF_BUS_WIDTH-1);
}SENDDDD_DBUF_WUSED_T;
typedef struct {		//SENDDDD_DBUF_T
	unsigned char Data[UART_SENDDDD_DBUF_SIZE];
	SENDDDD_DBUF_PTR_T pWrite;
	SENDDDD_DBUF_PTR_T pRead;
	SENDDDD_DBUF_WUSED_T Wused;			
	union  {		//Status
		unsigned char Val;
		struct 
		{
			__IO uint8_t	Full  : 1;
			__IO uint8_t	Empty : 1;
		} BITs;
	}Status;
} SENDDDD_DBUF_T;
//SENDDDD_LBUF
typedef struct {		//SENDDDD_LBUF_PTR_T
	uint16_t Val : UART_SENDDDD_LBUF_BUS_WIDTH;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_SENDDDD_LBUF_BUS_WIDTH);	
}SENDDDD_LBUF_PTR_T;
typedef struct {		//SENDDDD_LBUF_WUSED_T
	uint16_t Val : UART_SENDDDD_LBUF_BUS_WIDTH+1;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_SENDDDD_LBUF_BUS_WIDTH-1);
}SENDDDD_LBUF_WUSED_T;
typedef struct {		//SENDDDD_LBUF_T
	unsigned short Data[UART_SENDDDD_LBUF_SIZE];
	SENDDDD_LBUF_PTR_T pWrite;
	SENDDDD_LBUF_PTR_T pRead;
	SENDDDD_LBUF_WUSED_T Wused;
	union  {		//Status
		unsigned char Val;
		struct 
		{
			__IO uint8_t	Full  : 1;
			__IO uint8_t	Empty : 1;
		} BITs;
	}Status;
} SENDDDD_LBUF_T;
//RECEIVE_DBUF
typedef struct {		//RECEIVE_DBUF_PTR_T
	uint16_t Val : UART_RECEIVE_DBUF_BUS_WIDTH;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_RECEIVE_DBUF_BUS_WIDTH);	
}RECEIVE_DBUF_PTR_T;
typedef struct {		//RECEIVE_DBUF_WUSED_T
	uint16_t Val : UART_RECEIVE_DBUF_BUS_WIDTH+1;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_RECEIVE_DBUF_BUS_WIDTH-1);
}RECEIVE_DBUF_WUSED_T;
typedef struct {		//RECEIVE_DBUF_T
	unsigned char Data[UART_RECEIVE_DBUF_SIZE];
	RECEIVE_DBUF_PTR_T pWrite;
	RECEIVE_DBUF_PTR_T pRead;
	RECEIVE_DBUF_WUSED_T Wused;		
	union  {		//Status
		unsigned char Val;
		struct{
			__IO uint8_t	Full  : 1;
			__IO uint8_t	Empty : 1;
		} BITs;
	}Status;
} RECEIVE_DBUF_T;
//RECEIVE_LBUF
typedef struct {		//RECEIVE_LBUF_PTR_T
	uint16_t Val : UART_RECEIVE_LBUF_BUS_WIDTH;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_RECEIVE_LBUF_BUS_WIDTH);	
}RECEIVE_LBUF_PTR_T;
typedef struct {		//RECEIVE_LBUF_WUSED_T
	uint16_t Val : UART_RECEIVE_LBUF_BUS_WIDTH+1;
	uint16_t Reserve : (8*sizeof(uint16_t)-UART_RECEIVE_LBUF_BUS_WIDTH-1);
}RECEIVE_LBUF_WUSED_T;
typedef struct {		//RECEIVE_LBUF_T
	unsigned short Data[UART_RECEIVE_LBUF_SIZE];
	RECEIVE_LBUF_PTR_T pWrite;
	RECEIVE_LBUF_PTR_T pRead;
	RECEIVE_LBUF_WUSED_T Wused;		
	union  {		//Status
		unsigned char Val;
		struct{
			__IO uint8_t	Full  : 1;
			__IO uint8_t	Empty : 1;
		} BITs;
	}Status;
} RECEIVE_LBUF_T;

typedef struct {		//UART_T
	unsigned int baudrate;
	const unsigned int BAUDRATE_VALID[UART_BAUDRATE_VALID_NUM];
	unsigned int framelength;
	float chartime;
	float chartimex3P5;
	float chartimex4;
	
	SENDDDD_DBUF_T send_dbuf;
	SENDDDD_LBUF_T send_lbuf;
	void(*send_dbuf_wr)(unsigned short data);
	unsigned char(*send_dbuf_rd)(void);
	void(*send_lbuf_wr)(unsigned short data);
	unsigned char(*send_lbuf_rd)(void);


	
	RECEIVE_DBUF_T receive_dbuf;
	RECEIVE_LBUF_T receive_lbuf;
	void(*receive_dbuf_wr)(unsigned short data);
	unsigned char(*receive_dbuf_rd)(void);
	void(*receive_lbuf_wr)(unsigned short data);
	unsigned char(*receive_lbuf_rd)(void);
	unsigned short (*send_crc)(SENDDDD_DBUF_PTR_T phead, unsigned short length);
	unsigned short (*receive_crc)(RECEIVE_DBUF_PTR_T phead, unsigned short length);
	
	
	
	unsigned int send_length;
	float (*cal_chartime)(unsigned int baudrate);
	float (*cal_framelength)(void);

	void(*send_string_int)(void);							//Send String by Interrupt Mode
	void(*normalsend_string_int)(void);
	void(*modbusend_string_int)(void);												//Modebus Protocol: 4xCharTime + DATA + 4xCharTimes
	unsigned char(*init)(void);
	unsigned char(*dmainit)(void);
	void (*dmasend)(void);
	unsigned char(*setbaudrate)(unsigned int baudrate);
	int (*cfg)(ADI_UART_TypeDef *pPort, int iBaud, int iBits, int iFormat);
	void(*Int_Handler)(void);
	/*Uart Message Management*/
	enum {UART_IDLE = 0x00 , UART_MODBUS_RECEIVE = 0x01, UART_NORMAL_SEND = 0x02, UART_MODBUS_1stSILENCE = 0x03, UART_MODBUS_DATA = 0x04, UART_MODBUS_2ndSILENCE = 0x05} status;
}UART_T;


extern UART_T uart;





#endif	/*UART_H_ */
